/**
 * 電子メール送信ライブラリ J-mail ver 3.0 
 * Copyright (c) 2015～2020 Takashi Kawaba Released under the MIT license
 * https://opensource.org/licenses/MIT
 */
package jmail;

import java.io.Serializable;
import java.util.List;
import java.util.Properties;
import javax.annotation.PostConstruct;
import javax.enterprise.context.Dependent;
import javax.inject.Inject;
import javax.servlet.ServletContext;

/**
 * 平文、HTML送信、CC/BCC指定、ファイル添付での送信メソッドがある
 * 接続－送信（複数）－切断のシーケンスを個別に実行するためのメソッドもある
 * @author kawaba
 */
@Dependent
public class EmailSender implements Serializable {
    private Properties prop;        // 送信用パラメータ
    private String smtpUser;
    private String smtpPasswd;
    private String host;
    private int port;
    
    @Inject private ServletContext context;
    @Inject private JmSender js;    // Java Mailによるメール送信
    @Inject SenderProperties conf;      // パラメータの取得方法を実装している
    
    // 送信用パラメータを取得しておく
    @PostConstruct
    public void init(){
        prop = conf.getProperties();
        smtpUser = (String)prop.getProperty("smtpUser");
        smtpPasswd = (String)prop.getProperty("smtpPassword");
        host = (String)prop.getProperty("host");
        port = Integer.parseInt((String)prop.getProperty("port"));
    }

///////////////////////////////////////////////////////////////////////////////
//    A.接続－送信（複数）－切断のシーケンスを個別に実行するためのメソッド
///////////////////////////////////////////////////////////////////////////////    
/**
 * メールサーバーに接続する
 * @return 実行結果（EmailStatus型）
 */
public EmailStatus connect(){
    if(!js.init(smtpUser, smtpPasswd))  return EmailStatus.INIT_ERROR;
    if(!js.connect(host, port))         return EmailStatus.CONNECT_ERROR;
    return EmailStatus.DONE;
}
/**
 * １通の平文メールを送信する
 * @param to    宛先
 * @param subject メールタイトル
 * @param body   メール本文
 * @return 実行結果（EmailStatus型）
 */
public EmailStatus xsend(String to, String subject, String body){
    if(!js.send(to, smtpUser, subject, body)) return EmailStatus.SEND_ERROR;
    return EmailStatus.DONE;
}
/**
 * １通のHTMLメールを送信する
 * @param to    宛先
 * @param subject メールタイトル
 * @param body   メール本文
 * @return 実行結果（EmailStatus型）
 */
public EmailStatus xsendHtml(String to, String subject, String body){
    if(!js.sendHtml(to, smtpUser, subject, body)) return EmailStatus.SEND_ERROR;
    return EmailStatus.DONE;
}
/**
 * メールサーバーから切断する
 * @return 実行結果（EmailStatus型）
 */
public EmailStatus disconnect(){
    if(!js.disconnect()) return EmailStatus.DISCONNECT_ERROR;
    return  EmailStatus.DONE;
}
    
///////////////////////////////////////////////////////////////////////////////
//   B.平文メール（毎回、接続－送信－切断のシーケンスを実行する）
///////////////////////////////////////////////////////////////////////////////    
    /**
     * 1-10.SMTP-AUTH 送信
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param body   メール本文
     * @return 実行結果（EmailStatus型）
     */
    public EmailStatus send(String to, String subject, String body) {
        return send(prop, to, subject, body);
    }

    /**
     * 1-11.SMTP-AUTH 送信、CC-BCC指定("TO"、"CC"、"BCC")
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param body   メール本文
     * @param type  送信タイプ("TO"、"CC"、"BCC")
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus send(String to, String subject, String body, String type) {
        return send(prop, to, subject, body, type);
    }
    /**
     * 1-12.SMTP-AUTH 送信、ファイル添付
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param body   メール本文
     * @param fileDir   ファイルのあるディレクトリ
     * @param flist ファイル名のリスト
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus send( String to,  String subject, String body, String fileDir, List<String> flist) {
        return send(prop, to, subject, body, fileDir, flist);
    }

    /**
     * 1-13.SMTP-AUTH 送信、ファイル添付、CC-BCC指定("TO"、"CC"、"BCC")
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param body   メール本文
     * @param fileDir   ファイルのあるディレクトリ
     * @param flist ファイル名のリスト
     * @param type  送信タイプ("TO"、"CC"、"BCC")
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus send(String to,  String subject, String body, String fileDir, List<String> flist, String type) {
        return send(prop, to, subject, body, fileDir, flist, type);

    }  
///////////////////////////////////////////////////////////////////////////////
//    C.HTMLメール （毎回、接続－送信－切断のシーケンスを実行する）
///////////////////////////////////////////////////////////////////////////////

    /**
     * 1-20.SMTP-AUTH、HTML送信
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param htmlFile  HTML文（文字列）
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus sendHtml(String to, String subject, String htmlFile) {
        return sendHtml(prop, to, subject, htmlFile);
    }

    /**
     * 1-21.SMTP-AUTH、HTML送信、CC-BCC指定("TO"、"CC"、"BCC")
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param htmlFile  HTML文（文字列）
     * @param type  送信タイプ("TO"、"CC"、"BCC")
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus sendHtml(String to,  String subject, String htmlFile, String type) {
        return sendHtml(prop, to, subject, htmlFile, type);
    } 

    /**
     * 1-22.SMTP-AUTH、HTML送信、ファイル添付
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param htmlFile  HTML文（文字列）
     * @param fileDir   ファイルのあるディレクトリ
     * @param flist ファイル名のリスト
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus sendHtml(String to,  String subject, String htmlFile, String fileDir, List<String> flist) {
        return sendHtml(prop, to, subject, htmlFile, fileDir, flist);
    }

    /**
     * 1-23.MTP-AUTH、HTML送信、ファイル添付、CC-BCC指定("TO"、"CC"、"BCC")
     *
     * @param to    宛先
     * @param subject メールタイトル
     * @param htmlFile  HTML文（文字列）
     * @param fileDir   ファイルのあるディレクトリ
     * @param flist ファイル名のリスト
     * @param type  送信タイプ("TO"、"CC"、"BCC")
     * @return 実行結果（EmailStatus型）
     */
    public  EmailStatus sendHtml(String to,String subject, String htmlFile, String fileDir, List<String> flist, String type) {
        return sendHtml(prop, to, subject, htmlFile, fileDir, flist, type);
    } 
///////////////////////////////////////////////////////////////////////////////
//    セッター、ゲッター
///////////////////////////////////////////////////////////////////////////////

    /**
     * SMTPサーバーへの接続プロパティを得る
     * @return 接続プロパティ
     */
    public Properties getProp() {
        return prop;
    }
    /**
     * SMTPサーバーへの接続プロパティをセットする
     * @param prop 接続プロパティ
     */
    public void setProp(Properties prop) {
        this.prop = prop;
    }    
    
//////////////////////////////////////////////////////////////////////////////
//   以下のメソッドはサポートメソッドなので公開しない
//////////////////////////////////////////////////////////////////////////////
///    平文メール　プロパティ指定
///////////////////////////////////////////////////////////////////////////////    
    /**
     * 10.SMTP-AUTH 送信、Propertiesパラメータ
     *
     * @param prop メールサーバー情報
     * @param to
     * @param subject
     * @param body
     */
    private EmailStatus send(Properties prop, String to, String subject, String body) {
        return send(prop, to, subject, body, "TO");
    }

    /**
     * 11.SMTP-AUTH 送信、Propertiesパラメータ、CC-BCC指定
     *
     * @param prop
     * @param to
     * @param subject
     * @param body
     * @param type
     */
    private  EmailStatus send(Properties prop, String to, String subject, String body, String type) {
        return send((String) prop.get("smtpUser"), (String) prop.get("smtpPassword"), (String) prop.get("host"), Integer.parseInt((String) prop.get("port")), to, subject, body, type);
    }
    /**
     * 12.SMTP-AUTH 送信、Propertiesパラメータ、ファイル添付
     *
     * @param prop
     * @param to
     * @param subject
     * @param body
     * @param fileDir
     * @param flist
     */
    private  EmailStatus send(Properties prop, String to, String subject, String body, String fileDir, List<String> flist) {
        return send(prop, to, subject, body, fileDir, flist, "TO");

    }

    /**
     *13.SMTP-AUTH 送信、Propertiesパラメータ、ファイル添付、CC-BCC指定
     *
     * @param prop
     * @param to
     * @param subject
     * @param body
     * @param fileDir
     * @param flist
     * @param type
     */
    private  EmailStatus send(Properties prop, String to, String subject, String body, String fileDir, List<String> flist, String type) {
        return send((String) prop.get("smtpUser"), (String) prop.get("smtpPassword"), (String) prop.get("host"), Integer.parseInt((String) prop.get("port")), to, subject, body, fileDir, flist, type);

    }
///////////////////////////////////////////////////////////////////////////////
//    HTMLメール プロパティ指定
///////////////////////////////////////////////////////////////////////////////

    /**
     * 20.SMTP-AUTH、HTML送信、Propertiesパラメータ
     *
     * @param prop
     * @param to
     * @param subject
     * @param htmlFile
     */
    private  EmailStatus sendHtml(Properties prop, String to, String subject, String htmlFile) {
        return sendHtml(prop, to, subject, htmlFile, "TO");
    }

    /**
     * 21.SMTP-AUTH、HTML送信、Propertiesパラメータ、CC-BCC指定
     *
     * @param prop
     * @param to
     * @param subject
     * @param htmlFile
     * @param type
     */
    private  EmailStatus sendHtml(Properties prop, String to, String subject, String htmlFile, String type) {
        return sendHtml((String) prop.get("smtpUser"), (String) prop.get("smtpPassword"), (String) prop.get("host"), Integer.parseInt((String) prop.get("port")), to, subject, htmlFile, type);
    } 

    /**
     * 22.SMTP-AUTH、HTML送信、Propertiesパラメータ、ファイル添付
     *
     * @param prop
     * @param to
     * @param subject
     * @param htmlFile
     * @param fileDir
     * @param flist
     */
    private  EmailStatus sendHtml(Properties prop, String to, String subject, String htmlFile, String fileDir, List<String> flist) {
        return sendHtml(prop, to, subject, htmlFile, fileDir, flist, "TO");
    }

    /**
     * 23.MTP-AUTH、HTML送信、Propertiesパラメータ、ファイル添付、CC-BCC指定
     *
     * @param prop
     * @param to
     * @param subject
     * @param htmlFile
     * @param fileDir
     * @param flist
     * @param type
     */
    private  EmailStatus sendHtml(Properties prop, String to, String subject, String htmlFile, String fileDir, List<String> flist, String type) {
        return sendHtml((String) prop.get("smtpUser"), (String) prop.get("smtpPassword"), (String) prop.get("host"), Integer.parseInt((String) prop.get("port")), to, subject, htmlFile, fileDir, flist, type);
    }    
///////////////////////////////////////////////////////////////////////////////
//   平文Lメール 全パラメータ指定
///////////////////////////////////////////////////////////////////////////////   
/**
     * 30.SMTP-AUTH 送信
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param body
     */
    private  EmailStatus send(String smtpUser, String smtpPassword, String host, int port, String to,String subject, String body) {
        return send(smtpUser, smtpPassword, host, port, to, subject, body, "TO");
    }
    /**
     * 31.SMTP-AUTH 送信、CC-BCC指定
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param body
     * @param type
     */
    private   EmailStatus  send(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String body, String type) {
        if(!js.init(smtpUser, smtpPassword)) return EmailStatus.INIT_ERROR;
        if(!js.connect(host, port)) return EmailStatus.CONNECT_ERROR;
        if(!js.send(to, smtpUser, subject, body, type)) return EmailStatus.SEND_ERROR;
        if(!js.disconnect()) return EmailStatus.DISCONNECT_ERROR;
        return EmailStatus.DONE;
    }
    /**
     * 32.SMTP-AUTH 送信、ファイル添付
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param body
     * @param fileDir
     * @param flist
     */
    private  EmailStatus send(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String body, String fileDir, List<String> flist) {
        return send(smtpUser, smtpPassword, host, port, to, subject, body, fileDir, flist, "TO");
    }
    /**
     * 33.SMTP-AUTH 送信、ファイル添付、CC-BCC指定
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param body
     * @param fileDir
     * @param flist
     * @param type
     */
    private  EmailStatus send(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String body, String fileDir, List<String> flist, String type) {
        if(!js.init(smtpUser, smtpPassword)) return EmailStatus.INIT_ERROR;
        if(!js.connect(host, port)) return EmailStatus.CONNECT_ERROR;
        if(!js.send(to, smtpUser, subject, body, fileDir, flist, type)) return EmailStatus.SEND_ERROR;
        if(!js.disconnect()) return EmailStatus.DISCONNECT_ERROR;
        return EmailStatus.DONE;
    }
///////////////////////////////////////////////////////////////////////////////
//    HTMLメール　全パラメータ指定
///////////////////////////////////////////////////////////////////////////////
    /**
     * 40.SMTP-AUTH、HTML送信
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param htmlFile
     */
    private  EmailStatus sendHtml(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String htmlFile) {
        return sendHtml(smtpUser, smtpPassword, host, port, to, subject, htmlFile, "TO");
    }
    /**
     * 41.SMTP-AUTH、HTML送信、CC-BCC指定
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param htmlFile
     * @param type
     */
    private  EmailStatus sendHtml(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String htmlFile, String type) {
        if(!js.init(smtpUser, smtpPassword)) return EmailStatus.INIT_ERROR;
        if(!js.connect(host, port)) return EmailStatus.CONNECT_ERROR;
        if(!js.sendHtml(to, smtpUser, subject, htmlFile, type)) return EmailStatus.SEND_ERROR;
        if(!js.disconnect()) return EmailStatus.DISCONNECT_ERROR;
        return EmailStatus.DONE;
    }
    /**
     * 42.MTP-AUTH、HTML送信、ファイル添付
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param htmlFile
     * @param fileDir
     * @param flist
     */
    private  EmailStatus sendHtml(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String htmlFile, String fileDir, List<String> flist) {
        return sendHtml(smtpUser, smtpPassword, host, port, to, subject, htmlFile, fileDir, flist, "TO");
    }
    /**
     * 43.MTP-AUTH、HTML送信、ファイル添付、CC-BCC指定
     *
     * @param smtpUser
     * @param smtpPassword
     * @param host
     * @param port
     * @param to
     * @param subject
     * @param htmlFile
     * @param type
     * @param fileDir
     * @param flist
     */
    private  EmailStatus sendHtml(String smtpUser, String smtpPassword, String host, int port, String to, String subject, String htmlFile, String fileDir, List<String> flist, String type) {
        if(!js.init(smtpUser, smtpPassword)) return EmailStatus.INIT_ERROR;        
        if(!js.connect(host, port)) return EmailStatus.CONNECT_ERROR;
        if(!js.sendHtml(to, smtpUser, subject, htmlFile, fileDir, flist, type)) return EmailStatus.SEND_ERROR;
        if(!js.disconnect()) return EmailStatus.DISCONNECT_ERROR;
        return EmailStatus.DONE;
    }

 

}
